-- Routines for reading 3DStudio .asc files and compiling them into OpenGL display lists.
-- By Mic, 2000
--
-- Currently, only single-object .asc files are supported (multi-object files can be read
-- but only the first object will be loaded).
-- All material/shininess parameters are ignored. Instead, you have to specify all such
-- parameters.
--
-- Syntax:
-- procedure euglCompileAsc(sequence name,sequence params)
--
--  name = filename ("*.asc")
--  params =
--  {
--   atom 	list,			handle generated by glGenLists
--   sequence 	origin			origin of object ({x,y,z})
--   atom	size,			size of object (actually size/2)
--   sequence	material_ambient,	{r,g,b,a}
--   sequence	material_diffuse,	{r,g,b,a}
--   sequence	material_specular,	{r,g,b,a}
--   sequence	emission,		the light that the object sends out ({r,g,b,a})
--   atom	shininess,		shininess of material
--  }
--



without warning

include file.e
include wildcard.e
include get.e
include eugl.ew


constant ascf32mem = allocate(4*4)


sequence asc
integer x




function read_file(atom fn,atom dest,integer bytes)
	integer byte,x

 	byte = 0
 	x = 0
 	for i = 1 to bytes do
  		byte = getc(fn)
  		if byte = -1 then
   			return 1
  		end if
  		poke(dest+x,byte)
  		x += 1
 	end for
 	
 	return 0
end function
 
 

function get_int()
	sequence temp
 
 	temp = {'0'}
 	while (asc[x] >= '0') and (asc[x] <= '9') do
  		temp = temp & asc[x]
  		x += 1
 	end while
 	temp = value(temp)
 	if temp[1] != GET_SUCCESS then
  		return 0
 	end if

 	return temp[2]
end function



function get_float()
	sequence num

 	num={}
 	while (asc[x] != ' ') and (asc[x] != 13) do
  		if asc[x] = ',' then
   			num = num & '.'
  		else
   			num = num & asc[x]
  		end if
  		x += 1
 	end while
 	num = value(num)
 	if num[1] != GET_SUCCESS then
  		num[2] = 0
 	end if
 
 	return num[2]
end function



global function read_asc_file(sequence filename)
	atom fn
	integer byte,vertices,faces
	integer a,b,c,d
	sequence obj,crap
	
	fn = open(filename,"rb")
	if fn = -1 then
 		return {-1}
	end if

	x = 0
	asc = {}
	while x != -1 do
 		x = getc(fn)
 		if x = -1 then
  			exit
 		end if
 		asc = asc & x
	end while
	close(fn)

	x = 1
	while compare("Tri-mesh",asc[x..x+7]) do
 		x += 1
	end while
 
 	while compare("Vertices",asc[x..x+7]) do
  		x += 1
 	end while
 
 	crap="0"
 	x += 10
 	vertices = get_int()
 	while compare("Faces",asc[x..x+4]) do
  		x += 1
 	end while
 
 	crap = "0"
 	x += 7
 	faces = get_int()
 	obj={0,0}
 	obj[1] = repeat({0,0,0},vertices)
 	obj[2] = repeat({0,0,0},faces)
 
 	while compare("Vertex",asc[x..x+5]) do
  		x += 1
 	end while
 	x += 27
 	a = 1
 
 	for i = 1 to vertices do
  		obj[1][i][1] = get_float()
  		x += 8
  		obj[1][i][2] = get_float()
  		x += 8
  		obj[1][i][3] = get_float()
		--  x=x+15
  		if i < vertices then
   			while asc[x] != 'X' do
    				x += 1
   			end while
   			x += 3
  		end if
 	end for
 
 	x += 27
 	for i = 1 to faces do
  		obj[2][i][1] = get_int()+1
  		x += 3
  		obj[2][i][2] = get_int()+1
  		x += 3
  		obj[2][i][3] = get_int()+1
  		if i < faces then
   			while compare("Face",asc[x..x+3]) do
   	 			x += 1
   			end while
   			while asc[x] != 'A' do
    				x += 1
   			end while
   			x += 2
  		end if
 	end for
 
 	return obj
end function




global procedure euglCompileAsc(sequence name,sequence params)
	--params =
	--{
	-- list,
	-- {x,y,z},	origin
	-- size,
	-- {r,g,b},	ambient
	-- {r,g,b},	diffuse
	-- {r,g,b},	specular
	-- {r,g,b},	emission
	-- shininess
	--}

	atom miinx,miiny,miinz
	atom maaxx,maaxy,maaxz
	atom xadd,yadd,zadd
	atom miin,cur
	atom listptr
	sequence obj,faces

 	obj = read_asc_file(name)
 	if atom(obj[1]) then
  		return 
 	end if
 
 	faces = obj[2]
 	miin = 0
 	maaxx = -100000
 	maaxy = -100000
 	maaxz = -100000
 	miinx = 1000000
 	miiny = 1000000
 	miinz = 1000000
 
 	for i = 1 to length(obj[1]) do
  		cur = obj[1][i][1]
  		if cur < miinx then miinx = cur end if
  		if cur > maaxx then maaxx = cur end if
  		cur=obj[1][i][2]
  		if cur < miiny then miiny = cur end if
  		if cur > maaxy then maaxy = cur end if
  		cur=obj[1][i][3]
  		if cur < miinz then miinz = cur end if
  		if cur > maaxz then maaxz = cur end if
 	end for
 
 	xadd = ((miinx+maaxx)/2) + params[2][1]
 	yadd = ((miiny+maaxy)/2) + params[2][2]
 	zadd = ((miinz+maaxz)/2) + params[2][3]
 	for i = 1 to length(obj[1]) do
 		 obj[1][i][1] -= xadd
 		 obj[1][i][2] -= yadd
 		 obj[1][i][3] -= zadd
 	end for
 	for i = 1 to length(obj[1]) do
  		for j = 1 to 3 do
   			cur = obj[1][i][j]
   			if cur < 0 then cur = (-cur) end if
   			if cur > miin then miin = cur end if
  		end for
 	end for
 
 	obj = (obj[1]*(params[3]/miin))
 	for i = 1 to length(obj) do
  		obj[i][3] -= 1
 	end for

 	pokef32(ascf32mem,params[4])
 	c_proc(glMaterialfv,{ GL_FRONT_AND_BACK, GL_AMBIENT, ascf32mem })
 	pokef32(ascf32mem,params[5])
 	c_proc(glMaterialfv,{ GL_FRONT_AND_BACK, GL_DIFFUSE, ascf32mem })
 	pokef32(ascf32mem,params[6])
 	c_proc(glMaterialfv,{ GL_FRONT_AND_BACK, GL_SPECULAR, ascf32mem })
 	pokef32(ascf32mem,params[7])
 	c_proc(glMaterialfv,{ GL_FRONT_AND_BACK, GL_EMISSION, ascf32mem })
 	c_proc(glMaterialf,{ GL_FRONT_AND_BACK, GL_SHININESS, params[8] })

 	c_proc(glNewList, {params[1],GL_COMPILE })
 	c_proc(glBegin,{ GL_TRIANGLES })
 	for i=1 to length(faces) do
  		c_proc(glVertex3f,obj[faces[i][1]])
  		c_proc(glVertex3f,obj[faces[i][2]])
  		c_proc(glVertex3f,obj[faces[i][3]])
 	end for
 	c_proc(glEnd,{ })  
 	c_proc(glEndList,{ })
 
 	obj = {}
 	faces = {}
end procedure 